"""A Python 3 phonetics library."""
from .phonetics import (Soundex,
                        Metaphone,
                        MatchingRatingApproach,
                        FuzzySoundex,
                        Lein,
                        RefinedSoundex)

__version__ = '0.5.4'
